@@ -13,13 +13,13 @@ def get_generation(ba_name, **kwargs):
     # get data
     c = client_factory(ba_name)
     data = c.get_generation(**kwargs)
-    
+
     # log
     if len(data) == 0:
         msg = '%s: No generation data at %s with args %s' % (ba_name, datetime.utcnow().isoformat(),
                                                     kwargs)
         logger.warn(msg)
-    
+
     # return
     return data
 
@@ -28,13 +28,13 @@ def get_load(ba_name, **kwargs):
     # get data
     c = client_factory(ba_name)
     data = c.get_load(**kwargs)
-    
+
     # log
     if len(data) == 0:
         msg = '%s: No load data at %s with args %s' % (ba_name, datetime.utcnow().isoformat(),
                                                     kwargs)
         logger.warn(msg)
-    
+
     # return
     return data
 
@@ -44,12 +44,29 @@ def get_trade(ba_name, **kwargs):
     # get data
     c = client_factory(ba_name)
     data = c.get_trade(**kwargs)
-    
+
     # log
     if len(data) == 0:
         msg = '%s: No trade data at %s with args %s' % (ba_name, datetime.utcnow().isoformat(),
                                                     kwargs)
         logger.warn(msg)
-    
+
     # return
     return data
+
+@shared_task
+def get_lmp(ba_name, node_list, **kwargs):
+    # get data
+    c = client_factory(ba_name)
+    data = c.get_lmp(node_list, **kwargs)
+
+    # log
+    if len(data) == 0:
+        msg = '%s: No lmp data at %s with args %s' % (ba_name, datetime.utcnow().isoformat(),
+                                                    kwargs)
+        logger.warn(msg)
+
+    # return
+    return data
+
+
